#!/bin/bash
#
# Install script for Flashphoner WebCallServer
#



# Global variables configuration

# Declare global variables
function declareGlobals() {
  declareCommonGlobals
  INSTALLER_DIR=${INSTALLER_DIR:-$(pwd)}
  INSTALL_LOG=${INSTALL_LOG:-$INSTALLER_DIR/install.log}
  UPDATE_LOG=${UPDATE_LOG:-$WCS_HOME/logs/update.log}
  VERSION_WITH_HASH=${VERSION_WITH_HASH:-$(cat $INSTALLER_DIR/WCS.version)}
  VERSION=${VERSION:-$(echo $VERSION_WITH_HASH | cut -d- -f1)}
  WCS_HOME_FULL="$WCS_HOME-$VERSION"
}

# Get installer dir
function getInstallerDir() {
  echo "$INSTALLER_DIR"
}

# Get install log file
function getInstallLogFile() {
  echo "$INSTALL_LOG"
}

# Get update log file
function getUpdateLogFile() {
  echo "$UPDATE_LOG"
}

# Get version to install
function getVersion() {
  echo "$VERSION"
}

# Get version with hash to install
function getVersionWithHash() {
  echo "$VERSION_WITH_HASH"
}

# Get major version to install
function getMajorVersion() {
  local majorVersion=$(echo $VERSION | cut -d. -f1)

  echo "$majorVersion"
}

# Get minor version to install
function getMinorVersion() {
  local minorVersion=$(echo $VERSION | cut -d. -f2)

  echo "$minorVersion"
}

# Get full installation path including version
function getFullHome() {
  echo "$WCS_HOME_FULL"
}


# Common global variables configuration

# Declare common global variables
function declareCommonGlobals() {
  PRODUCT=FlashphonerWebCallServer
  LOCAL_DIR=/usr/local
  WCS_HOME="$LOCAL_DIR/$PRODUCT"
  FLASHPHONER_USER="flashphoner"
  SILENT_MODE=${SILENT_MODE:-false}
}

# Get product name
function getProduct() {
  echo "$PRODUCT"
}

# Get local directory to install
function getLocal() {
  echo "$LOCAL_DIR"
}

# Get WCS home directory
function getHome() {
  echo "$WCS_HOME"
}

# Get WCS user name
function getUser() {
  echo "$FLASHPHONER_USER"
}

# Is installation non-interactive
function isSilent() {
  $SILENT_MODE; return
}

# Set install to be non-interactive
function setSilent() {
  SILENT_MODE=true
}


# Common user interface functions

# Display a message to console if allowed
function informUser() {
  local message=$1

  if ! isSilent; then
    echo -e "$message"
  fi
}

# Read user input
function askUser() {
  local message=$@
  local prompt=""
  local input=""

  if [[ ! -z $message ]]; then
    read -p "$message " input < /dev/tty
  else
    read input < /dev/tty
  fi
  echo "$input"
}


# Logging functions

# Declare logging variables
function declareLog() {
  local logFile=$1

  DEBUG=${DEBUG:-false}
  DEFAULT_LOG=${DEFAULT_LOG:-}

  setDefaultLogFile $logFile
}

# Check if debug log enabled
function isDebugEnabled() {
  if [[ ! -z $DEBUG ]]; then
    $DEBUG; return
  else
    false; return
  fi
}

# Enable debug logging
function enableDebug() {
  DEBUG=true
}

# Disable debug logging
function disableDebug() {
  DEBUG=false
}

# Get default log file
function getDefaultLogFile() {
  echo "$DEFAULT_LOG"
}

# Set default log file
function setDefaultLogFile() {
  local logFile=$1

  [[ -z $logFile ]] && return 1
  if checkLogPath $logFile; then
    DEFAULT_LOG=$logFile
  fi
}

# # Check if log file path exists and make the dir if needed
function checkLogPath() {
  local logFile=$1
  local result=0

  [[ -z $logFile ]] && return 1
  logPath=$(dirname "$logFile")
  if [[ ! -d $logPath ]]; then
    mkdir -p $logPath
    result=$?
  fi
  return $result
}

# Form log timestamp
function logDate() {
  local dateToLog=`echo "[$(date '+%Y-%m-%d %H:%M:%S')]"`
 
  echo "$dateToLog"
}

# Write to a log file
function writeLog() {
  local level=$1
  local message=$2
  local logFile=$3
  local owner=$(getCurrentUser)
  local logPath=""
  local callerFuncName=""

  [[ -z $logFile ]] && logFile=$(getDefaultLogFile)
  if [[ "$level" == "" || "$message" == "" || "$logFile" == "" ]]; then
    return 1
  fi
  if [[ -f $logFile ]]; then
    owner=$(stat -c "%U" $logFile)
  else
    checkLogPath $logFile
    [[ $? -ne 0 ]] && return 1
  fi
  # Determine caller function name
  for callerFuncName in ${FUNCNAME[@]}; do
    if [[ "$callerFuncName" == "${FUNCNAME[0]}" || "$callerFuncName" =~ ^log ]]; then
      continue
    else
      break
    fi
  done
  if [[ "$owner" != "$(getCurrentUser)" && "$(command -v sudo 2>/dev/null)" != "" ]]; then
    # The log file is owned by someone else, try to sudo to it if sudo available
    sudo -u $owner echo -e "$(logDate) $level $callerFuncName - $message" >> $logFile
  else
    echo -e "$(logDate) $level $callerFuncName - $message" >> $logFile
  fi
 }

# Write INFO level message
function logInfo() {
  local message=$1
  local logFile=$2

  writeLog "INFO" "$message" "$logFile"
}

# Write WARN level message
function logWarn() {
  local message=$1
  local logFile=$2

  writeLog "WARN" "$message" "$logFile"
}

# Write ERROR level message
function logError() {
  local message=$1
  local logFile=$2

  writeLog "ERROR" "$message" "$logFile"
}

# Write DEBUG level message
function logDebug() {
  local message=$1
  local logFile=$2

  if isDebugEnabled; then
    writeLog "DEBUG" "$message" "$logFile"
  fi
}


# Various helper functions

# Check if user exists in system
function isUser() {
  local user=$1
  
  if [[ -z $user ]]; then
    user=$(getUser)
  fi
  if [[ "$(getent passwd $user)" =~ ^$user ]]; then
    true; return
  else
    false; return
  fi
}

# Get current user running script
function getCurrentUser() {
  echo "$(whoami)"
}

# Get daemon type
function getDaemonType() {
  local daemonType=`ps --no-headers -o comm 1`
  echo "$daemonType"
} 

# Check if service is running
function isService() {
  # Check service only if systemd used #WCS-3034
  if [[ $(getDaemonType) != 'systemd' ]]; then
    false; return
  fi
  if systemctl is-active --quiet webcallserver.service; then
    true; return
  else
    false; return
  fi
}

# Check if the script is running from root
function isScriptRunningFromRoot() {
  if [[ "$(getCurrentUser)" == "root" ]]; then
    true; return
  else
    false; return
  fi
}

# Set file system item owner to user
function setOwner() {
  local user=$1
  local object=$2

  if [[ "$user" == "" || "$object" == "" ]]; then
    return 1
  fi
  [[ ! isScriptRunningFromRoot ]] && return 1
  [[ ! -e $object ]] && return 1
  chown -RHL $user:$(id -gn $user) $object > /dev/null 2>&1
}

# Get Java version
function getJavaVersion() {
  local javaCmd=`command -v java 2>/dev/null`
  local jdkVersion=""
  local javaFullVersion=""
  local javaMajor=0
  local javaMinor=0
  local javaVersion=0

  if [[ -z $javaCmd ]]; then
    echo "$javaVersion"
    return 1
  fi
  jdkVersion=`$javaCmd -version 2>&1`
  javaFullVersion=`echo $jdkVersion | head -1 | cut -d" " -f 3 | tr -d \"`
  javaMajor=`echo $javaFullVersion | cut -d \. -f 1`
  if [[ $javaMajor -eq 1 ]]; then
    javaMinor=`echo $javaFullVersion | cut -d \. -f 2`
    if [[ $javaMinor -ge 8  ]]; then
      javaVersion=$javaMinor
    fi
  elif [[ $javaMajor -ge 8 ]]; then
    javaVersion=$javaMajor
  fi
  logDebug "Java version $javaVersion"
  echo "$javaVersion"
}

# Get user running the process by pid
function getUserByPid() {
  local pid=$1
  local user=`ps -o uname= -p "$pid"`
 
  logDebug "Process $pid running as user $user"
  echo "$user"
}

# Check if user is owner of the object
function isOwned() {
  local object=$1
  local user=$2
  local owner=""
  
  if [[ -z $user ]]; then
    user=$(getUser)
  fi
  owner=$(stat -c "%U" $object)
  if [[ $owner == $user ]]; then
    true; return
  else
    false; return
  fi
}

# Get next file copy name as file.0, file.1 etc
function getNextCopyName() {
  local pattern=$1
  local lastFile=""
  local lastSuffix=""
  local newSuffix=0
  local nextCopyName=""
  
  if [[ -z $pattern ]]; then
    echo ""
    return 1
  fi
  lastFile=`ls $pattern.[0-9]* | sort -Vr | head -1`
  [[ -z $lastFile ]] && lastFile=`ls $pattern | sort -Vr | head -1`
  [[ -z $lastFile ]] && lastFile=$pattern
  lastSuffix=`echo $lastFile | grep -o '[^.]*$'`
  logDebug "lastSuffix: $lastSuffix"
  if [[ $lastSuffix =~ ^[0-9]+$ ]]; then
    newSuffix=$((lastSuffix+1))
  else
    lastSuffix=""
  fi
  logDebug "lastSuffix: $lastSuffix, newSuffix: $newSuffix"
  if [[ ! -z $lastSuffix ]]; then
    nextCopyName=`echo $lastFile | sed 's/\(.*\)'$lastSuffix'/\1'$newSuffix'/'`
  else
    nextCopyName=$lastFile.$newSuffix
  fi
  
  echo "$nextCopyName"
}

# Check if user can write the object
function isWritable() {
  local object=$1
  local user=$2
  local asUser=""
  
  if [[ -z $user ]]; then
    user=$(getUser)
  fi
  if [[ "$(getCurrentUser)" != $user ]]; then
    asUser="sudo -u $user"
  fi
  if $asUser test -w $object; then
    true; return
  else
    false; return
  fi
}


# Functions to manage WCS process

# Try to stop service
function tryStopService() {
  local pidFile=$1

  [[ -z $pidFile ]] && return 1
  if [[ -f $pidFile && $(pgrep -F $pidFile) ]]; then
    if [[ "$(getDaemonType)" == "systemd" ]]; then
      systemctl stop webcallserver > /dev/null 2>&1
    else
      service webcallserver stop > /dev/null 2>&1
    fi
  fi
}

# Try to stop application
function tryStopApp() {
  local pidFile=$1

  [[ -z $pidFile ]] && return 1
  if [[ -f $pidFile && $(pgrep -F $pidFile) ]]; then
    informUser "Service did not stop, stopping application"
    $(getHome)/bin/webcallserver stop > /dev/null 2>&1
  fi
}

# Try to stop process
function tryStopProcess() {
  local pidFile=$1
  local pid=""
  local msgForceStop=""

  pid=$(pgrep -fn com.flashphoner.server.Server)
  if [[ ! -z $pid ]]; then
    msgForceStop="Server process found (pid $pid), stopping forcefully"
    informUser "$msgForceStop"
    logWarn "$msgForceStop"
    kill -9 $pid
    [[ ! -z $pidFile && -f $pidFile ]] && rm -rf $pidFile
  fi
}


# Stop running instance
function stopRunningServer() {
  local pidFile=$1
  local msgCantStop=""

  [[ -z $pidFile ]] && return 1
  tryStopService $pidFile
  sleep 1
  tryStopApp $pidFile
  sleep 1
  if [[ -f $pidFile && $(pgrep -F $pidFile) ]]; then
    msgCantStop="Cannot stop Java process $(pgrep -F $pidFile), please kill it forcefully"
    informUser "ERROR: $msgCantStop"
    logError "$msgCantStop"
    return 1
  fi
  # if still there is a process, try to kill it
  tryStopProcess
  return 0
}



# Message displaying functions

# Display EULA
function displayLicense() {
  local pageFile=""
  local page=""
  local answer=""

  if isSilent || isInstalled; then
    return 0
  fi

  for pageFile in `ls EULA_PAGE* | sort`; do
    page=`cat $pageFile`
    echo "$page"
    echo ""
    echo "Press ENTER if you agree to the license terms. Press Ctrl+C to stop installation."
    answer="$(askUser)"
  done

  until [[ "$answer" == "yes" || "$answer" == "no" ]]; do
    answer="$(askUser 'Do you agree to the license terms? [yes/no]')"
  done
  clear
  if [[ "$answer" == "no" ]]; then
    false; return
  fi
  true; return
}

# Display installation result
function displayResult() {
  local input=""
  local msgIns="Installation complete!"
  local msgUpd="Update complete!      "
  local msgFinish="$msgIns"
  local msgUpdated=""

  if isInstalled; then
    msgFinish="$msgUpd"
    msgUpdated="Updated from $(getPreviousVersion) to $(getVersion)"
    logInfo "$msgUpdated"
    logInfo "$msgUpdated" $(getUpdateLogFile)
  else
    logInfo "$msgFinish"
  fi
  if isSilent; then
    return 0
  fi
  informUser "
************************************************************************************************
*                                                                                              *
*                                   $msgFinish                                     *
*                             Thank you for trying Flashphoner!                                *
*                                                                                              *
*               Please restart $(getProduct) before start the work.                 *
*                  Write us with any questions to support@flashphoner.com                      *
*                                                                                              *
*                                                                                              *
*                                                                                              *
************************************************************************************************
"
  askUser
}

# Prerequisites checking functions

# Check if Java installed
function checkJava() {
  local javaVersion=$(getJavaVersion)
  local msgOldJava=""
  local msgJava="Java detected successfully, version"

  informUser "DETECTING Java version..."
  if [[ $javaVersion -eq 0 ]]; then
    informUser "
ERROR: The Java command (java) could not be found.
Search path: $PATH
In most cases this problem can be fixed by adding a symbolic 
link to the Java command in the /usr/bin directory. 
To do this first execute the command \"which java\" to identify 
the full path to the Java executable. Next, create a symbolic
link to this file with the command
\"ln -sf [path-to-java] /usr/bin/java\" where [path-to-java] is
the path returned by the \"which\" command.
"
    logError "No Java found!"
    false; return
  elif [[ $javaVersion -lt 8 ]]; then
    msgOldJava="Java version is $javaVersion, must be at least 8!"
    informUser "ERROR: $msgOldJava"
    logError "$msgOldJava"
    false; return
  else
    msgJava="$msgJava $javaVersion"
    informUser "- $msgJava"
    logInfo "$msgJava"
    true; return
  fi
}

# Get Java architecture id (32 or 64)
function getJavaArchId() {
  local javaCmd=`command -v java 2>/dev/null`
  local jdkVersion=""
  local javaArchId=0

  if [[ -z $javaCmd ]]; then
    echo "$javaArchId"
    return 1
  fi
  jdkVersion=`$javaCmd -version 2>&1`
  javaArchId=`echo $jdkVersion | grep 64 | sed 's/\(.*\)\(64\)\(.*\)/\2/'`
  if [ "$javaArchId" != "64" ]; then
    javaArchId="32"
  fi
  echo "$javaArchId"
}

# Get hardware platform full id
function getHwPlatform() {
  local hwPlatform=`uname -m`

  echo "$hwPlatform"
}

# Get hardware platform id (32 or 64)
function getHwPlatformId() {
  local hwPlatformId=`echo $(getHwPlatform) | sed 's/\(.*\)\(64\)\(.*\)/\2/'`

  if [ "$hwPlatformId" != "64" ]; then
    hwPlatformId="32"
  fi
  echo "$hwPlatformId"
}

# Check prerequisites main function
function checkPrerequisites() {
  local javaArchId=0
  local hwPlatform="$(getHwPlatform)"
  local hwPlatformId=0
  local msgNotCompatible=""

  informUser "
******************************************************
*                                                    *
*              Checking the system                   * 
*                                                    *                                                     
******************************************************
"
  if ! checkJava; then
    false; return
  fi
  informUser "DETECTING JVM architecture..."
  javaArchId="$(getJavaArchId)"
  informUser "- $javaArchId bit architecture detected"
  informUser "DETECTING hardware platform..."
  hwPlatformId="$(getHwPlatformId)"
  informUser "- $hwPlatformId bit hardware platform detected"
  if [[ "$hwPlatformId" != "$javaArchId" ]]; then
    msgNotCompatible="JVM architecture $javaArchId is not compatible with hardware architecture $hwPlatformId bit($hwPlatform)!"
    informUser "
ERROR. $msgNotCompatible
Please, uninstall current JVM/JDK and download and install JDK $hwPlatform.
Support - support@flashphoner.com, forum - www.flashphoner.com/forums
"
    logError "$msgNotCompatible"
    false; return
  fi
  informUser "
******************************************************
*                                                    *
*       The system is ready for installation.        *
*                                                    *
*                                        Press ENTER *
******************************************************
"
  ! isSilent && askUser
  true; return
}


# Functions to check previous version

# Declare version globals
function declareVersion() {
  ALREADY_INSTALLED=${ALREADY_INSTALLED:-false}
  PRODUCT_VERSION_FILE=${PRODUCT_VERSION_FILE:-$(getHome)/conf/WCS.version}
  PREVIOUS_VERSION=${PREVIOUS_VERSION:-}
  PREVIOUS_VERSION_WITH_HASH=${PREVIOUS_VERSION_WITH_HASH:-}
  PREVIOUS_VERSION_DIR=${PREVIOUS_VERSION_DIR:-}
}

# Is server already installed
function isInstalled() {
  $ALREADY_INSTALLED; return
}

# Set server installed flag
function setInstalled() {
  ALREADY_INSTALLED=true
}

# Get installed version file location
function getInstalledVersionFile() {
  echo "$PRODUCT_VERSION_FILE"
}

# Get previous version
function getPreviousVersion() {
  echo "$PREVIOUS_VERSION"
}

# Get previous version with hash
function getPreviousVersionWithHash() {
  echo "$PREVIOUS_VERSION_WITH_HASH"
}

# Get previous major version
function getPreviousMajorVersion() {
  local previousMajorVersion=$(echo $PREVIOUS_VERSION | cut -d. -f1)

  echo "$previousMajorVersion"
}

# Get previous minor version
function getPreviousMinorVersion() {
  local previousMinorVersion=$(echo $PREVIOUS_VERSION | cut -d. -f2)

  echo "$previousMinorVersion"
}

# Set previuos version
function setPreviousVersion() {
  local version=$1

  [[ -z $version ]] && return 1
  PREVIOUS_VERSION_WITH_HASH=$version
  PREVIOUS_VERSION=$(echo $version | cut -d- -f1)
}

# Get previous version full home dir
function getPreviousFullHome() {
  echo "$PREVIOUS_VERSION_DIR"
}

# Set previuos version full home dir
function setPreviousFullHome() {
  local dir=$1

  [[ -z $dir ]] && return 1
  PREVIOUS_VERSION_DIR=$dir
}

# Check previous instance state and stop if running
function stopPreviousInstance() {
  local pidFilePath="/var/run"
  local pidFileName="FlashphonerMainWebCallServer.pid"
  local pidFile=""
  local envFile="$(getHome)/bin/setenv.sh"
  local confirm="yes"
  local result=0

  if [[ -f $envFile && $(grep "WCS_NON_ROOT=true" $envFile) ]]; then
    pidFilePath="$(getHome)/bin"
  fi
  pidFile="$pidFilePath/$pidFileName"
  logDebug "Get pid from file: $pidFile"
  if [[ -f $pidFile && $(pgrep -F $pidFile) ]]; then
    informUser "\nYou have to stop $(getProduct) before update\n"
    ! isSilent && confirm="$(askUser 'Stop now [yes/no] ? ')"
    if [[ $confirm == y* ]]; then
      stopRunningServer $pidFile
      result=$?
    else
      informUser "\nAbort updating\n "
      result=1
    fi
  fi
  return $result
}

# Main previous version check function
function checkPreviousVersion() {
  local previousVersionFile=""
  local previousVersion=""
  local previousFullHome=""
  local msgAlreadyInstalled=""
  local homeDir=$(getHome)
  local msgBanner=""
  local installed=false

  declareVersion
  previousVersionFile=$(getInstalledVersionFile)
  if [[ -f $previousVersionFile ]]; then
    previousVersion=$(cat $previousVersionFile)
    previousFullHome=$(readlink -e $homeDir)
    logDebug "Previous version found: $previousVersion"
    setPreviousVersion $previousVersion
    setPreviousFullHome $previousFullHome
    setInstalled
  fi
  if [[ "$(getPreviousVersionWithHash)" == "$(getVersionWithHash)" ]]; then
    msgAlreadyInstalled="Flashphoner v.$(getVersion) already installed"
    informUser "
******************************************************
*                                                    *
*    $msgAlreadyInstalled     *
*                                                    *
******************************************************
"
    logInfo "$msgAlreadyInstalled"
    ! isSilent && askUser
    false; return
  fi
  if isInstalled; then
    ! stopPreviousInstance && return 1
    msgBanner="Update Flashphoner from $(getPreviousVersion) to $(getVersion)"
  else
    msgBanner="     Installing Flashphoner v.$(getVersion)"
  fi
  informUser "
******************************************************
*                                                    *
*  $msgBanner  
*                                                    *
*  (C) Flashphoner.com 2010. All rights reserved.    *
*  To install press ENTER, to abort press CTRL+C.    *
*                                                    *
*                                        Press ENTER *
******************************************************
"
  ! isSilent && askUser
  true; return
}

# Check if updates allowed
function updatesAllowed() {
  local addOptions=""
  local javaVersion=$(getJavaVersion)

  if ! isInstalled; then
    true; return
  fi
  [[ $javaVersion -eq 16 ]] && addOptions="--illegal-access=permit"
  [[ $javaVersion -ge 17 ]] && addOptions="--add-exports java.base/sun.security.provider=ALL-UNNAMED --add-opens java.base/java.lang=ALL-UNNAMED"
  java -Dcom.flashphoner.fms.AppHome="$(getHome)" $addOptions -cp "./server/lib/*" com.flashphoner.server.license.ExpiredUpdates
  if [ $? -eq 1 ]; then
    false; return
  fi
  true; return
}


# Installation functions

# Copy Web SDK with examples
function copySdk() {
  local fullHome=$(getFullHome)
  local xmlFile=$fullHome/client/examples/flashphoner.xml

  cd $(getInstallerDir)
  cp -R client $fullHome
  cp -R client2 $fullHome
  sed -i s/87\.226\.225\.[0-9][0-9]/192.168.1.5/g $xmlFile
  sed -i s/192.168.1.[0-9]/127.0.0.1/g $xmlFile
}

# Update bin folder content
function updateBin() {
  local fullHome=$(getFullHome)
  local installerDir=$(getInstallerDir)
  local filesToKeep=(
    "on_multiple_record_hook.sh"
    "on_record_hook.sh"
    "setenv.sh"
  )
  local file=""

  cd $installerDir/server
  if [[ "$(getMajorVersion).$(getMinorVersion)" == "$(getPreviousMajorVersion).$(getPreviousMinorVersion)" ]]; then
    # Preserve possible scripts tweaks while updating the same product branch #WCS-2615
    for file in bin/*; do
      if [[ " ${filesToKeep[@]} " =~ " $(basename ${file}) " ]]; then
        [[ ! -f "$fullHome/$file" ]] && cp -f $file $fullHome/$file
      else
        cp -f $file $fullHome/$file
      fi
    done    
  else
    cp -R bin $fullHome
  fi
}

# Copy bin folder
function copyBin() {
  local fullHome=$(getFullHome)
  local installerDir=$(getInstallerDir)

  cd $installerDir/server
  mkdir -p $fullHome/bin
  if ! isInstalled; then
    cp -R bin $fullHome
  else
    updateBin
  fi
  cp -f $installerDir/uninstall.sh $fullHome/bin
  chmod +x $fullHome/bin/*.sh
  chmod +x $fullHome/bin/webcallserver
}

# Copy web folder
function copyWeb() {
  cd $(getInstallerDir)/server
  cp -R web $(getFullHome)
}

# Copy lib folder
function copyLib() {
  cd $(getInstallerDir)/server
  cp -R lib $(getFullHome)
}

# Copy tools folder
function copyTools() {
  local fullHome=$(getFullHome)

  cd $(getInstallerDir)/server
  cp -R tools $fullHome
  chmod +x $fullHome/tools/dump_sessions.sh
  chmod +x $fullHome/tools/certbot-auto
  chmod +x $fullHome/tools/report.sh
  chmod +x $fullHome/tools/pcap_*
  chmod +x $fullHome/tools/offline_mixer_tool.sh
  chmod +x $fullHome/tools/mixer_layout_tool.sh
}

# Update conf folder
function updateConf() {
  local configs=(
    "log4j.properties"
    "accounts.xml"
    "database.yml"
    "account.xml"
    "callee.xml"
    "crossdomain.xml"
    "token_keys.properties"
    "dtls-main-cert.pem"
    "dtls-main-key.pem"
    "watchdog.properties"
    "wcs-core.properties"
    "wss.jks.backup"
    "cli-hostkey.pem"
    "dtls0_ua"
    "offline-mixer.json"
  )
  local subdirs=(
    "qa"
    "apps/click-to-call"
  )
  local fullHome=$(getFullHome)
  local installerDir=$(getInstallerDir)
  local item=""

  for item in ${configs[@]}; do
    if [[ ! -f $fullHome/conf/$item && -f $installerDir/server/conf/$item ]]; then
      cp $installerDir/server/conf/$item $fullHome/conf
    fi
    if [[ $item == "database.yml" ]]; then
      sed -i 's/9091/8081\/apps/g' $fullHome/conf/database.yml
    fi
  done
  for item in ${subdirs[@]}; do
    if [[ ! -d "$fullHome/conf/$item" ]]; then
      cp -R $installerDir/server/conf/$item $fullHome/conf
    fi
  done
}

# Copy conf folder
function copyConf() {
  local fullHome=$(getFullHome)
  local installerDir=$(getInstallerDir)

  cd $installerDir/server
  mkdir -p $fullHome/conf
  if ! isInstalled; then
    cp -R conf $fullHome
  else
    updateConf
  fi
}

# Make symlink to product folder
function makeSymlink() {
  local linkHome=$(getHome)
  local fullHome=$(getFullHome)

  if [ -L $linkHome ]; then
    rm -f $linkHome
  fi
  ln -sf $fullHome $linkHome
}

# Update version file
function updateVersion() {
  echo "$(getVersionWithHash)" > $(getHome)/conf/WCS.version
}

# Copying product files
function copyFiles() {
  local folders=(
    Sdk
    Bin
    Web
    Lib
    Tools
    Conf
  )
  local folder=""

  informUser "COPYING files..."
  for folder in ${folders[@]}; do
    copy$folder
  done
  makeSymlink
  updateVersion
  informUser "- Copying completed."
}

# Main installation function
function doInstall() {
  local fullHome="$(getFullHome)"

  if isInstalled; then
    informUser "Starting $(getProduct) update..."
    mv $(getPreviousFullHome) $fullHome
    rm -rf $fullHome/lib
  else
    informUser "
*************************************************************
*                                                           *
*      Starting $(getProduct) installation      *  
*                                                           *
*************************************************************
"
    [ ! -d $fullHome ] && mkdir $fullHome
  fi

  copyFiles
}

# Functions to prepare non-root user

# Create user if not exists
function prepareUser() {
  local user=$(getUser)

  if ! isUser $user; then
    useradd --system --home-dir $(getHome) --user-group $user
  fi
}


# Functions to install or update service

# Declere service globals
function declareService() {
  SYSTEMD_PATH=${SYSTEMD_PATH:-/etc/systemd/system}
  INITD_PATH=${INITD_PATH:-/etc/init.d}
  SYSTEMD_FILE=${SYSTEMD_FILE:-$SYSTEMD_PATH/webcallserver.service}
  INITD_FILE=${INITD_FILE:-$INITD_PATH/webcallserver}
}

# Get systemd path
function getSystemdPath() {
  echo "$SYSTEMD_PATH"
}

# Get initd path
function getInitdPath() {
  echo "$SYSTEMD_PATH"
}

# Get systemd full file name
function getSystemdFileName() {
  echo "$SYSTEMD_FILE"
}

# Get initd full file name
function getInitdFileName() {
  echo "$INITD_FILE"
}

# Copy service setup file to systemd
function copyToSystemd() {
  cp $(getInstallerDir)/scripts/systemd/webcallserver.service $(getSystemdPath)
}

# Copy startup script to initd
function copyToInitd() {
  cp -f $(getFullHome)/bin/webcallserver $(getInitdFileName)
}

# Helper function tro ask user for run WCS on startup
function askToRunOnStartup() {
  local answer=""

  until [[ "$answer" == "yes" || "$answer" == "no" ]]; do
    answer=$(askUser 'Do you want to start WebCallServer at server startup? [yes/no]')
  done
  echo "$answer"
}

# Enable WCS running on startup
function startupEnable() {
  local msgCantDetectDistro="Unable to determine your distribution"

  if [[ "$(getDaemonType)" == "systemd" ]]; then
    systemctl enable webcallserver > /dev/null 2>&1
  elif [ ! -f /proc/version ]; then
    informUser "ERROR: $msgCantDetectDistro"
    logError "$msgCantDetectDistro"
    return 1
  elif grep -e "Red Hat\|centos" /proc/version > /dev/null; then
    chkconfig --add webcallserver > /dev/null 2>&1
    chkconfig --level 345 webcallserver on > /dev/null 2>&1
  elif grep -e "[Dd]ebian\|[Uu]buntu" /proc/version > /dev/null; then
    update-rc.d -f webcallserver remove > /dev/null 2>&1
    update-rc.d webcallserver defaults > /dev/null 2>&1
  else
    informUser "ERROR: $msgCantDetectDistro"
    logError "$msgCantDetectDistro"
    return 1
  fi
}

# Install service
function installService() {
  local runOnStartup="no"
  local startupEnabled=false

  if [[ "$(getDaemonType)" == "systemd" ]]; then
    copyToSystemd
    systemctl daemon-reload
  else
    copyToInitd
  fi
  ! isSilent && runOnStartup=$(askToRunOnStartup)
  if [[ "$runOnStartup" == "yes" ]]; then
    startupEnable && startupEnabled=true
  fi
  if ! $startupEnabled; then
    informUser "Use '$(getHome)/bin/webcallserver start' to start server"
  fi
}

# Update systemd service
function updateSystemd() {
  local systemdFile=$(getSystemdFileName)

  if [[ -f $systemdFile ]]; then
    # Add 143 exit code as success #WCS-3034
    if ! grep -e "^SuccessExitStatus" $systemdFile > /dev/null; then
      sed -i 's/\[Service\]/\[Service\]\nSuccessExitStatus=143\n/g' $systemdFile
    fi
    # Revert user back to root if there is such settings,
    # service should always be started from root with sudoing to flashphoner #WCS-3112
    sed -i -e "s/^\(User[ \t]*=\).*\$/\1root/" $systemdFile
    sed -i -e "s/^\(Group[ \t]*=\).*\$/\1root/" $systemdFile
  else
    copyToSystemd
  fi
  systemctl daemon-reload
}

# Update service
function updateService() {
  if [[ "$(getDaemonType)" == "systemd" ]]; then
    updateSystemd
  elif [[ -f $(getInitdFileName) ]]; then
    copyToInitd
  fi
}

# Main service setup function
function serviceSetup() {
  declareService
  if ! isInstalled; then
    installService
  else
    updateService
  fi
}



# Main module

# Usage displaying
function usage() {
  echo "Usage: $(basename $0) [OPTIONS]"
  echo -e "  -silent | --silent\tInstall WCS in non interactive mode (do not ask anything)"
  return 1
}

# Main script function
function main() {
  local msgRestricted=""
  local msgNotAllowed=""

  declareGlobals
  declareLog $(getInstallLogFile)

  # Parse command line arguments
  while [[ $# -gt 0 ]]; do
    case $1 in
      --debug)
        enableDebug
        shift
        ;;
      -silent|--silent)
        setSilent
        shift
        ;;
      *)
        usage
        return 1
        ;;
    esac
  done

  # Check write permissions
  if [ ! -w "$(getLocal)" ]; then
    msgRestricted="Write to $(getLocal) restricted for user running install.sh"
    informUser "ERROR: $msgRestricted"
    logError "$msgRestricted"
    return 1
  fi

  if ! checkPrerequisites; then
    return 1
  fi
  if ! checkPreviousVersion; then
    return 1
  fi
  if isInstalled && [[ ! updatesAllowed ]]; then
    msgNotAllowed="Updates are not allowed for version $(getPreviousVersion)"
    informUser "ERROR: $msgNotAllowed"
    logError "$msgNotAllowed"
    return 1    
  fi
  if ! displayLicense; then
    return 1
  fi
  doInstall
  prepareUser
  serviceSetup
  displayResult 
  if isUser $(getUser); then
    setOwner $(getUser) $(getFullHome)
  fi
}

main "$@"

exit $?


