#!/bin/bash
#
# Uninstall script for Flashphoner WebCallServer
#



# Global variables configuration

# Declare global variables
function declareGlobals() {
  declareCommonGlobals
  INSTALL_LOG="$WCS_HOME/bin/install.log"
  VERSION_WITH_HASH=$(cat $WCS_HOME/conf/WCS.version)
  VERSION=$(echo $VERSION_WITH_HASH | cut -d- -f1)
  WCS_HOME_FULL=$(readlink -e $WCS_HOME)
}

# Get install log file
function getInstallLogFile() {
  if [[ -f $INSTALL_LOG ]]; then
    echo "$INSTALL_LOG"
  else
    echo ""
  fi
}

# Get full installation path including version
function getFullHome() {
  if [[ -d $WCS_HOME_FULL ]]; then
    echo "$WCS_HOME_FULL"
  else
    echo ""
  fi
}


# Common global variables configuration

# Declare common global variables
function declareCommonGlobals() {
  PRODUCT=FlashphonerWebCallServer
  LOCAL_DIR=/usr/local
  WCS_HOME="$LOCAL_DIR/$PRODUCT"
  FLASHPHONER_USER="flashphoner"
  SILENT_MODE=${SILENT_MODE:-false}
}

# Get product name
function getProduct() {
  echo "$PRODUCT"
}

# Get local directory to install
function getLocal() {
  echo "$LOCAL_DIR"
}

# Get WCS home directory
function getHome() {
  echo "$WCS_HOME"
}

# Get WCS user name
function getUser() {
  echo "$FLASHPHONER_USER"
}

# Is installation non-interactive
function isSilent() {
  $SILENT_MODE; return
}

# Set install to be non-interactive
function setSilent() {
  SILENT_MODE=true
}


# Common user interface functions

# Display a message to console if allowed
function informUser() {
  local message=$1

  if ! isSilent; then
    echo -e "$message"
  fi
}

# Read user input
function askUser() {
  local message=$@
  local prompt=""
  local input=""

  if [[ ! -z $message ]]; then
    read -p "$message " input < /dev/tty
  else
    read input < /dev/tty
  fi
  echo "$input"
}


# Logging functions

# Declare logging variables
function declareLog() {
  local logFile=$1

  DEBUG=${DEBUG:-false}
  DEFAULT_LOG=${DEFAULT_LOG:-}

  setDefaultLogFile $logFile
}

# Check if debug log enabled
function isDebugEnabled() {
  if [[ ! -z $DEBUG ]]; then
    $DEBUG; return
  else
    false; return
  fi
}

# Enable debug logging
function enableDebug() {
  DEBUG=true
}

# Disable debug logging
function disableDebug() {
  DEBUG=false
}

# Get default log file
function getDefaultLogFile() {
  echo "$DEFAULT_LOG"
}

# Set default log file
function setDefaultLogFile() {
  local logFile=$1

  [[ -z $logFile ]] && return 1
  if checkLogPath $logFile; then
    DEFAULT_LOG=$logFile
  fi
}

# # Check if log file path exists and make the dir if needed
function checkLogPath() {
  local logFile=$1
  local result=0

  [[ -z $logFile ]] && return 1
  logPath=$(dirname "$logFile")
  if [[ ! -d $logPath ]]; then
    mkdir -p $logPath
    result=$?
  fi
  return $result
}

# Form log timestamp
function logDate() {
  local dateToLog=`echo "[$(date '+%Y-%m-%d %H:%M:%S')]"`
 
  echo "$dateToLog"
}

# Write to a log file
function writeLog() {
  local level=$1
  local message=$2
  local logFile=$3
  local owner=$(getCurrentUser)
  local logPath=""
  local callerFuncName=""

  [[ -z $logFile ]] && logFile=$(getDefaultLogFile)
  if [[ "$level" == "" || "$message" == "" || "$logFile" == "" ]]; then
    return 1
  fi
  if [[ -f $logFile ]]; then
    owner=$(stat -c "%U" $logFile)
  else
    checkLogPath $logFile
    [[ $? -ne 0 ]] && return 1
  fi
  # Determine caller function name
  for callerFuncName in ${FUNCNAME[@]}; do
    if [[ "$callerFuncName" == "${FUNCNAME[0]}" || "$callerFuncName" =~ ^log ]]; then
      continue
    else
      break
    fi
  done
  if [[ "$owner" != "$(getCurrentUser)" && "$(command -v sudo 2>/dev/null)" != "" ]]; then
    # The log file is owned by someone else, try to sudo to it if sudo available
    sudo -u $owner echo -e "$(logDate) $level $callerFuncName - $message" >> $logFile
  else
    echo -e "$(logDate) $level $callerFuncName - $message" >> $logFile
  fi
 }

# Write INFO level message
function logInfo() {
  local message=$1
  local logFile=$2

  writeLog "INFO" "$message" "$logFile"
}

# Write WARN level message
function logWarn() {
  local message=$1
  local logFile=$2

  writeLog "WARN" "$message" "$logFile"
}

# Write ERROR level message
function logError() {
  local message=$1
  local logFile=$2

  writeLog "ERROR" "$message" "$logFile"
}

# Write DEBUG level message
function logDebug() {
  local message=$1
  local logFile=$2

  if isDebugEnabled; then
    writeLog "DEBUG" "$message" "$logFile"
  fi
}


# Various helper functions

# Check if user exists in system
function isUser() {
  local user=$1
  
  if [[ -z $user ]]; then
    user=$(getUser)
  fi
  if [[ "$(getent passwd $user)" =~ ^$user ]]; then
    true; return
  else
    false; return
  fi
}

# Get current user running script
function getCurrentUser() {
  echo "$(whoami)"
}

# Get daemon type
function getDaemonType() {
  local daemonType=`ps --no-headers -o comm 1`
  echo "$daemonType"
} 

# Check if service is running
function isService() {
  # Check service only if systemd used #WCS-3034
  if [[ $(getDaemonType) != 'systemd' ]]; then
    false; return
  fi
  if systemctl is-active --quiet webcallserver.service; then
    true; return
  else
    false; return
  fi
}

# Check if the script is running from root
function isScriptRunningFromRoot() {
  if [[ "$(getCurrentUser)" == "root" ]]; then
    true; return
  else
    false; return
  fi
}

# Set file system item owner to user
function setOwner() {
  local user=$1
  local object=$2

  if [[ "$user" == "" || "$object" == "" ]]; then
    return 1
  fi
  [[ ! isScriptRunningFromRoot ]] && return 1
  [[ ! -e $object ]] && return 1
  chown -RHL $user:$(id -gn $user) $object > /dev/null 2>&1
}

# Get Java version
function getJavaVersion() {
  local javaCmd=`command -v java 2>/dev/null`
  local jdkVersion=""
  local javaFullVersion=""
  local javaMajor=0
  local javaMinor=0
  local javaVersion=0

  if [[ -z $javaCmd ]]; then
    echo "$javaVersion"
    return 1
  fi
  jdkVersion=`$javaCmd -version 2>&1`
  javaFullVersion=`echo $jdkVersion | head -1 | cut -d" " -f 3 | tr -d \"`
  javaMajor=`echo $javaFullVersion | cut -d \. -f 1`
  if [[ $javaMajor -eq 1 ]]; then
    javaMinor=`echo $javaFullVersion | cut -d \. -f 2`
    if [[ $javaMinor -ge 8  ]]; then
      javaVersion=$javaMinor
    fi
  elif [[ $javaMajor -ge 8 ]]; then
    javaVersion=$javaMajor
  fi
  logDebug "Java version $javaVersion"
  echo "$javaVersion"
}

# Get user running the process by pid
function getUserByPid() {
  local pid=$1
  local user=`ps -o uname= -p "$pid"`
 
  logDebug "Process $pid running as user $user"
  echo "$user"
}

# Check if user is owner of the object
function isOwned() {
  local object=$1
  local user=$2
  local owner=""
  
  if [[ -z $user ]]; then
    user=$(getUser)
  fi
  owner=$(stat -c "%U" $object)
  if [[ $owner == $user ]]; then
    true; return
  else
    false; return
  fi
}

# Get next file copy name as file.0, file.1 etc
function getNextCopyName() {
  local pattern=$1
  local lastFile=""
  local lastSuffix=""
  local newSuffix=0
  local nextCopyName=""
  
  if [[ -z $pattern ]]; then
    echo ""
    return 1
  fi
  lastFile=`ls $pattern.[0-9]* | sort -Vr | head -1`
  [[ -z $lastFile ]] && lastFile=`ls $pattern | sort -Vr | head -1`
  [[ -z $lastFile ]] && lastFile=$pattern
  lastSuffix=`echo $lastFile | grep -o '[^.]*$'`
  logDebug "lastSuffix: $lastSuffix"
  if [[ $lastSuffix =~ ^[0-9]+$ ]]; then
    newSuffix=$((lastSuffix+1))
  else
    lastSuffix=""
  fi
  logDebug "lastSuffix: $lastSuffix, newSuffix: $newSuffix"
  if [[ ! -z $lastSuffix ]]; then
    nextCopyName=`echo $lastFile | sed 's/\(.*\)'$lastSuffix'/\1'$newSuffix'/'`
  else
    nextCopyName=$lastFile.$newSuffix
  fi
  
  echo "$nextCopyName"
}

# Check if user can write the object
function isWritable() {
  local object=$1
  local user=$2
  local asUser=""
  
  if [[ -z $user ]]; then
    user=$(getUser)
  fi
  if [[ "$(getCurrentUser)" != $user ]]; then
    asUser="sudo -u $user"
  fi
  if $asUser test -w $object; then
    true; return
  else
    false; return
  fi
}


# Functions to manage WCS process

# Try to stop service
function tryStopService() {
  local pidFile=$1

  [[ -z $pidFile ]] && return 1
  if [[ -f $pidFile && $(pgrep -F $pidFile) ]]; then
    if [[ "$(getDaemonType)" == "systemd" ]]; then
      systemctl stop webcallserver > /dev/null 2>&1
    else
      service webcallserver stop > /dev/null 2>&1
    fi
  fi
}

# Try to stop application
function tryStopApp() {
  local pidFile=$1

  [[ -z $pidFile ]] && return 1
  if [[ -f $pidFile && $(pgrep -F $pidFile) ]]; then
    informUser "Service did not stop, stopping application"
    $(getHome)/bin/webcallserver stop > /dev/null 2>&1
  fi
}

# Try to stop process
function tryStopProcess() {
  local pidFile=$1
  local pid=""
  local msgForceStop=""

  pid=$(pgrep -fn com.flashphoner.server.Server)
  if [[ ! -z $pid ]]; then
    msgForceStop="Server process found (pid $pid), stopping forcefully"
    informUser "$msgForceStop"
    logWarn "$msgForceStop"
    kill -9 $pid
    [[ ! -z $pidFile && -f $pidFile ]] && rm -rf $pidFile
  fi
}


# Stop running instance
function stopRunningServer() {
  local pidFile=$1
  local msgCantStop=""

  [[ -z $pidFile ]] && return 1
  tryStopService $pidFile
  sleep 1
  tryStopApp $pidFile
  sleep 1
  if [[ -f $pidFile && $(pgrep -F $pidFile) ]]; then
    msgCantStop="Cannot stop Java process $(pgrep -F $pidFile), please kill it forcefully"
    informUser "ERROR: $msgCantStop"
    logError "$msgCantStop"
    return 1
  fi
  # if still there is a process, try to kill it
  tryStopProcess
  return 0
}



# Message displaying functions

# Display uninstall banner
function displayUninstallBanner() {
  if isSilent; then
    return 0
  fi

  informUser "
**************************************************
*                                                *
*             Uninstalling Flashphoner           *
*                                                *
* (c) Flashphoner.com 2010. All rights reserved  *
*                                                *
**************************************************
"
  return 0
}

# Ask user to confirm uninstallation
function askForConfirmation() {
  local answer=""
  local message="Are you sure you want to uninstall $(getProduct)? [yes/no]"

  if isSilent; then
    true; return
  fi
  until [[ "$answer" == "yes" || "$answer" == "no" ]]; do
    answer=$(askUser $message)
  done
  if [[ "$answer" == "yes" ]]; then
    true; return
  fi
  false; return
}

# Display uninstallation result
function displayUninstallResult() {
  if isSilent; then
    return 0
  fi
  informUser "
***************************************************************************
*                                                                         *
*                         Uninstallation complete!                        *
*                                                                         *
*  Thank you for trying Flashphoner!                                      *
*  Please restart server before further work                              *
*  Support - support@flashphoner.com, forum - www.flashphoner.com/forums  *
*  Press Enter to continue                                                *
*                                                                         *
***************************************************************************
"
  askUser
}

# Functions to check server state and stop it

# Check previous instance state and stop if running
function stopRunningInstance() {
  local pidFilePath="/var/run"
  local pidFileName="FlashphonerMainWebCallServer.pid"
  local pidFile=""
  local envFile="$(getHome)/bin/setenv.sh"
  local confirm="yes"
  local result=0

  if [[ -f $envFile && $(grep "WCS_NON_ROOT=true" $envFile) ]]; then
    pidFilePath="$(getHome)/bin"
  fi
  pidFile="$pidFilePath/$pidFileName"
  logDebug "Get pid from file: $pidFile"
  if [[ -f $pidFile && $(pgrep -F $pidFile) ]]; then
    informUser "\nYou have to stop $(getProduct) before uninstall\n"
    ! isSilent && confirm="$(askUser 'Stop now [yes/no] ? ')"
    if [[ $confirm == y* ]]; then
      stopRunningServer $pidFile
      result=$?
    else
      informUser "\nAbort uninstallation\n "
      result=1
    fi
  fi
  return $result
}


# Functions to check and deactivate license

# Check if license is active and deactivate it
function deactivateLicense() {
  if [ -f $(getHome)/conf/flashphoner.license ]; then
	  informUser "Active license found, deactivating"
	  $(getHome)/bin/deactivation.sh > /dev/null 2>&1
  fi
}


# File removing functions

# Remove legacy version files (for backward compatibility reason)
function removeOldFiles() {
  local installLogFile=$(getInstallLogFile)
  local line=""

  [[ -z $installLogFile ]] && return 0
  if [[ -f $installLogFile ]]; then
    informUser "Removing $(getProduct) files according to install.log..."
    cat $installLogFile | while read line; do
      [ -f $line ] && rm -rf $line
    done
    informUser "- Removing old files completed."
  fi
}

# Remove product folder with symlink
function removeProductFolder() {
  cd /
  rm -f $(getHome)
  rm -rf $(getFullHome)
}

# Main files removing function
function removeFiles() {
  removeOldFiles
  removeProductFolder
}


# Functions to remove non-root user

# Remove user if exists
function removeUser() {
  local user=$(getUser)

  if isUser $user; then
    pkill -TERM -u $user
    userdel -f $user
  fi
}



# Functions to remove service

# Declere service globals
function declareService() {
  SYSTEMD_PATH=${SYSTEMD_PATH:-/etc/systemd/system}
  INITD_PATH=${INITD_PATH:-/etc/init.d}
  SYSTEMD_FILE=${SYSTEMD_FILE:-$SYSTEMD_PATH/webcallserver.service}
  INITD_FILE=${INITD_FILE:-$INITD_PATH/webcallserver}
}

# Get systemd path
function getSystemdPath() {
  echo "$SYSTEMD_PATH"
}

# Get initd path
function getInitdPath() {
  echo "$SYSTEMD_PATH"
}

# Get systemd full file name
function getSystemdFileName() {
  echo "$SYSTEMD_FILE"
}

# Get initd full file name
function getInitdFileName() {
  echo "$INITD_FILE"
}

# Remove service file from systemd
function removeFromSystemd() {
  if [[ -f $(getSystemdFileName) ]]; then
    rm -rf $(getSystemdFileName)
  fi
}

# Remove startup script from initd
function removeFromInitd() {
  if [[ -f $(getInitdFileName) ]]; then
    rm -rf $(getInitdFileName)
  fi
}

# Disable service startup
function disableService() {
  local msgCantDetectDistro="Unable to determine your distribution"

  if [ ! -f /proc/version ]; then
    informUser "ERROR: $msgCantDetectDistro"
    logError "$msgCantDetectDistro"
    return 1
  elif grep -e "Red Hat\|centos" /proc/version > /dev/null; then
    chkconfig --del webcallserver > /dev/null 2>&1
  elif grep -e "[Dd]ebian\|[Uu]buntu" /proc/version > /dev/null; then
    update-rc.d -f webcallserver remove > /dev/null 2>&1
  else
    informUser "ERROR: $msgCantDetectDistro"
    logError "$msgCantDetectDistro"
    return 1
  fi
}

# Disable and remove service
function removeService() {
  declareService
  if [[ "$(getDaemonType)" == "systemd" ]]; then
    systemctl disable webcallserver > /dev/null 2>&1
    removeFromSystemd
    systemctl daemon-reload
  else
    removeFromInitd
    if ! disableService; then
      return 1
    fi
  fi
}


# Main module

# Usage displaying
function usage() {
  echo "Usage: $(basename $0)"
  echo -e "  -silent | --silent\tUninstall WCS in non interactive mode (do not ask anything)"
  return 1
}

# Main script function
function main() {
  local msgRestricted=""
  local msgNotAllowed=""

  declareGlobals

  # Parse command line arguments
  while [[ $# -gt 0 ]]; do
    case $1 in
      -silent|--silent)
        setSilent
        shift
        ;;
      *)
        usage
        return 1
        ;;
    esac
  done

  # Check write permissions
  if [ ! -w "$(getLocal)" ]; then
    msgRestricted="Write to $(getLocal) restricted for user running uninstall.sh"
    informUser "ERROR: $msgRestricted"
    logError "$msgRestricted"
    return 1
  fi

  displayUninstallBanner
  if ! askForConfirmation; then
    return 1
  fi
  if ! stopRunningInstance; then
    return 1
  fi
  deactivateLicense
  informUser "UNINSTALLING $(getProduct)..."
  removeService
  removeFiles
  removeUser
  displayUninstallResult 
}

main "$@"

exit $?


